/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.drill.exec.rpc;

import com.codahale.metrics.Gauge;
import io.netty.buffer.PooledByteBufAllocatorL;
import org.apache.drill.exec.memory.BufferAllocator;
import org.apache.drill.exec.metrics.DrillMetrics;

public abstract class AbstractRpcMetrics implements RpcMetrics {
  //private static final org.slf4j.Logger logger = org.slf4j.LoggerFactory.getLogger(AbstractRpcMetrics.class);

  protected boolean useEncryptedCounter;

  public static final String CONNECTION_COUNTER_PREFIX = "drill.connections.rpc.";

  public static final String ALLOCATOR_METRICS_PREFIX = PooledByteBufAllocatorL.METRIC_PREFIX + "rpc.";

  protected void registerAllocatorMetrics(final BufferAllocator allocator, final String metricPrefix) {
    DrillMetrics.register(metricPrefix + "used", new Gauge<Long>() {
      @Override
      public Long getValue() {
        return allocator.getAllocatedMemory();
      }
    });

    DrillMetrics.register(metricPrefix + "peak", new Gauge<Long>() {
      @Override
      public Long getValue() {
        return allocator.getPeakMemoryAllocation();
      }
    });
  }

  public abstract void initialize(boolean useEncryptedCounter, BufferAllocator allocator);

}